@@ -183,7 +183,7 @@ function(X, Z, XX=NULL, BTE=c(2000,7000,2), R=1, m0r1=FALSE, linburn=FALSE,
            Ds2x = double(Ds2x * nnprime),
            improv = double(as.logical(improv) * nnprime),
            irank = integer(as.logical(improv) * nnprime),
-           ess = double(1),
+           ess = double(1 + itemps[1]*2),
            gpcs = double(4),
            sens.ZZ.mean = double(ngrid*d),
            sens.ZZ.q1 = double(ngrid*d),
