@@ -55,8 +55,8 @@ using namespace std;
  * constructor function
  */
 
-Matern::Matern(unsigned int col, Base_Prior *base_prior)
-  : Corr(col, base_prior)
+Matern::Matern(unsigned int dim, Base_Prior *base_prior)
+  : Corr(dim, base_prior)
 {
 
   /* sanity checks */
@@ -123,8 +123,9 @@ Corr& Matern::operator=(const Corr &c)
   assert(prior->CorrModel() == MATERN);
   assert(prior == ((Gp_Prior*) base_prior)->CorrPrior());
   
-  /* copy the covariance matrices */
-  Cov(e);
+  /* copy the covariance matrices -- no longer performed due 
+     the new economy argument in Gp/Base */
+  // Cov(e);
   
   return *this;
 }
@@ -157,17 +158,48 @@ void Matern::Init(double *dmat)
   NugInit(dmat[0], ! (bool) dmat[2]);
 }
 
+/*
+ * Jitter:
+ *
+ * fill jitter[ ] with the variance inflation factor.  That is,
+ * the variance for an observation with covariates in the i'th
+ * row of X will be s2*(1.0 + jitter[i]).  In standard tgp, the
+ * jitter is simply the nugget.  But for calibration and mr tgp,
+ * the jitter value depends upon X (eg real or simulated data).
+ * 
+ */
+
+double* Matern::Jitter(unsigned int n1, double **X)
+{
+  double *jitter = new_vector(n1);
+  for(unsigned int i=0; i<n1; i++) jitter[i] = nug;
+  return(jitter);
+}
+
+/*
+ * CorrDiag:
+ *
+ * Return the diagonal of the corr matrix K corresponding to X
+ *
+ */
+
+double* Matern::CorrDiag(unsigned int n1, double **X)
+{
+  double *corrdiag = new_vector(n1);
+  for(unsigned int i=0; i<n1; i++) corrdiag[i] = 1.0 + nug;
+  return(corrdiag);
+}
 
 /* 
- * DrawNug:
+ * DrawNugs:
  * 
  * draw for the nugget; 
  * rebuilding K, Ki, and marginal params, if necessary 
  * return true if the correlation matrix has changed; false otherwise
  */
 
-bool Matern::DrawNug(unsigned int n, double **X, double **F, double *Z, double *lambda, 
-		     double **bmu, double **Vb, double tau2, double itemp, bool cart, 
+bool Matern::DrawNugs(unsigned int n, double **X, double **F, double *Z, double *lambda, 
+		     double **bmu, double **Vb, double tau2, double itemp,
 		     void *state)
 {
   bool success = false;
@@ -184,7 +216,7 @@ bool Matern::DrawNug(unsigned int n, double **X, double **F, double *Z, double *
 		    Kchol_new, &log_det_K_new, &lambda_new, Vb_new, bmu_new, 
 		    gp_prior->get_b0(), gp_prior->get_Ti(), gp_prior->get_T(), tau2, 
 		    prior->NugAlpha(), prior->NugBeta(), gp_prior->s2Alpha(), 
-		    gp_prior->s2Beta(), (int) linear, itemp, (int) cart, state);
+		    gp_prior->s2Beta(), (int) linear, itemp, state);
   
   /* did we accept the draw? */
   if(nug_new != nug) { nug = nug_new; success = true; swap_new(Vb, bmu, lambda); }
@@ -210,7 +242,7 @@ void Matern::Update(unsigned int n, double **X)
     xDISTx = new_matrix(n, n);
     nd = n;
   }
-  dist_symm(xDISTx, col-1, X, n, PWR);
+  dist_symm(xDISTx, dim, X, n, PWR);
   matern_dist_to_K_symm(K, xDISTx, d, nu, bk, nb, nug, n);
   //delete_matrix(xDISTx);
 }
@@ -227,7 +259,7 @@ void Matern::Update(unsigned int n, double **K, double **X)
 {
    
   double ** xDISTx = new_matrix(n, n);
-  dist_symm(xDISTx, col-1, X, n, PWR);
+  dist_symm(xDISTx, dim, X, n, PWR);
   matern_dist_to_K_symm(K, xDISTx, d, nu, bk, nb, nug, n);
   delete_matrix(xDISTx);
 }
@@ -244,7 +276,7 @@ void Matern::Update(unsigned int n1, unsigned int n2, double **K, double **X, do
 {
   
   double **xxDISTx = new_matrix(n2, n1);
-  dist(xxDISTx, col-1, XX, n1, X, n2, PWR);
+  dist(xxDISTx, dim, XX, n1, X, n2, PWR);
   matern_dist_to_K(K, xxDISTx, d, nu, bk, nb, nug, n1, n2);
   delete_matrix(xxDISTx);
 }
@@ -260,7 +292,7 @@ void Matern::Update(unsigned int n1, unsigned int n2, double **K, double **X, do
 
 int Matern::Draw(unsigned int n, double **F, double **X, double *Z, 
 		 double *lambda, double **bmu, double **Vb, double tau2, 
-		 double itemp, bool cart, void *state)
+		 double itemp, void *state)
 {
   int success = 0;
   bool lin_new;
@@ -268,7 +300,7 @@ int Matern::Draw(unsigned int n, double **F, double **X, double *Z,
 
   /* sometimes skip this Draw for linear models for speed */
   if(linear && runi(state) > 0.5) 
-    return DrawNug(n, X, F, Z, lambda, bmu, Vb, tau2, itemp, cart, state);
+    return DrawNugs(n, X, F, Z, lambda, bmu, Vb, tau2, itemp, state);
 
   /* proppose linear or not */
   if(prior->Linear()) lin_new = true;
@@ -287,7 +319,7 @@ int Matern::Draw(unsigned int n, double **F, double **X, double *Z,
       xDISTx = new_matrix(n, n);
       nd = n;
     }
-    dist_symm(xDISTx, col-1, X, n, PWR);
+    dist_symm(xDISTx, dim, X, n, PWR);
     allocate_new(n); 
     assert(n == this->n);
   }
@@ -303,7 +335,7 @@ int Matern::Draw(unsigned int n, double **F, double **X, double *Z,
 			   gp_prior->get_b0(), gp_prior->get_Ti(), gp_prior->get_T(), tau2, 
 			   nug, nu, bk, nb, q_bak/q_fwd, ep->DAlpha(), ep->DBeta(), 
 			   gp_prior->s2Alpha(), gp_prior->s2Beta(), (int) lin_new, itemp, 
-			   (int) cart, state);
+			    state);
   }
   
   /* did we accept the new draw? */
@@ -318,7 +350,7 @@ int Matern::Draw(unsigned int n, double **F, double **X, double *Z,
   if(dreject >= REJECTMAX) return -2;
   
   /* draw nugget */
-  bool changed = DrawNug(n, X, F, Z, lambda, bmu, Vb, tau2, itemp, cart, state);
+  bool changed = DrawNugs(n, X, F, Z, lambda, bmu, Vb, tau2, itemp, state);
   success = success || changed;
   
   /* return true if anything has changed about the corr matrix */
@@ -551,7 +583,7 @@ double* Matern::Trace(unsigned int* len)
 
 Corr* Matern_Prior::newCorr(void)
 {
-  return new Matern(col, base_prior);
+  return new Matern(dim, base_prior);
 }
 
 
@@ -562,7 +594,7 @@ Corr* Matern_Prior::newCorr(void)
  * the exponential correlation function
  */
 
-Matern_Prior::Matern_Prior(unsigned int col) : Corr_Prior(col)
+Matern_Prior::Matern_Prior(unsigned int dim) : Corr_Prior(dim)
 {
   corr_model = MATERN;
 
@@ -823,7 +855,7 @@ void Matern_Prior::Draw(Corr **corr, unsigned int howmany, void *state)
   }
   
   /* hierarchical prior draws for the nugget */
-  DrawNug(corr, howmany, state);
+  DrawNugHier(corr, howmany, state);
 }
 
 
